'use strict';
const {Readable} = require('stream');

const kDidRead = Symbol('didRead');
const kDetached = Symbol('detached');

class HTTP2IncomingMessage extends Readable {
	constructor(req, response) {
		super();

		this.statusCode = response.headers[':status'];
		this.statusMessage = '';
		this.httpVersion = '2.0';
		this.httpVersionMajor = 2;
		this.httpVersionMinor = 0;
		this.headers = response.headers;
		this.trailers = {};
		this.req = req;

		this.aborted = false;
		this.complete = false;
		this.upgrade = req.method === 'CONNECT';

		this.rawHeaders = response.rawHeaders;
		this.rawTrailers = [];

		this.socket = req.socket;
		this.connection = req.socket;

		const self = this;

		this.on('end', () => {
			if (self.req.aborted) {
				self.aborted = true;

				// This event is usually emitted when the socket is closed,
				// but HTTP2 session can be reused for another requests.
				self.emit('aborted');
			} else {
				self.complete = true;
			}
		});

		this[kDidRead] = false;
		this[kDetached] = req.method === 'CONNECT';

		this._dumped = false;
	}

	_read() {
		// We don't need duplicated data.
		if (!this[kDidRead] && !this[kDetached]) {
			this[kDidRead] = true;
			const self = this;

			this.req._request.on('data', chunk => {
				/* istanbul ignore next */
				if (!self.push(chunk)) {
					self.req._request.pause();
				}
			});
		}
	}

	_destroy(error) {
		this.req._request.destroy(error);
	}

	setTimeout(ms, callback) {
		this.req.setTimeout(ms, callback);
		return this;
	}

	_dump() {
		/* istanbul ignore next */
		if (!this._dumped) {
			this._dumped = true;

			if (!this[kDetached]) {
				this.req._request.removeAllListeners('data');
				this.req._request.resume();
			}

			this.removeAllListeners('data');
			this.resume();
		}
	}
}

module.exports = HTTP2IncomingMessage;
