'use strict';
/* istanbul ignore file: https://github.com/nodejs/node/blob/master/lib/internal/errors.js */

module.exports = {};

const makeError = (Base, key, getMessage) => {
	module.exports[key] = class NodeError extends Base {
		constructor(...args) {
			super(typeof getMessage === 'string' ? getMessage : getMessage(args));
		}

		get name() {
			return `${super.name} [${key}]`;
		}

		set name(value) {
			Object.defineProperty(this, 'name', {
				configurable: true,
				enumerable: true,
				value,
				writable: true
			});
		}

		get code() {
			return key;
		}

		set code(value) {
			Object.defineProperty(this, 'code', {
				configurable: true,
				enumerable: true,
				value,
				writable: true
			});
		}
	};
};

makeError(Error, 'ERR_HTTP2_HEADERS_SENT', args => {
	if (args && args[0]) {
		return `Cannot ${args[0]} headers after they are sent to the client`;
	}

	return 'Response has already been initated.';
});

makeError(TypeError, 'ERR_INVALID_HTTP_TOKEN', args => {
	return `${args[0]} must be a valid HTTP token ["${args[1]}"]`;
});

makeError(TypeError, 'ERR_HTTP2_INVALID_HEADER_VALUE', args => {
	return `Invalid value "${args[0]}" for header "${args[1]}"`;
});

makeError(Error, 'ERR_HTTP2_NO_SOCKET_MANIPULATION', 'HTTP/2 sockets should not be directly manipulated (e.g. read and written)');

module.exports.SOCKET_HANG_UP = class NodeError extends Error {
	constructor() {
		super('socket hang up');

		this.code = 'ECONNRESET';
	}
};
