'use strict';
/* istanbul ignore file: remove this file after #2 is fixed */
const {ERR_HTTP2_NO_SOCKET_MANIPULATION} = require('./errors');

const kSavedSocket = Symbol('savedSocket');

// https://github.com/nodejs/node/blob/78584b64d88c0567c46e3cf7bae42aa73927df40/lib/internal/http2/core.js#L672

const proxySocketHandler = {
	get(session, prop) {
		switch (prop) {
			case 'setTimeout':
				return session.setTimeout.bind(session);
			case 'destroy':
			case 'emit':
			case 'end':
			case 'pause':
			case 'read':
			case 'resume':
			case 'write':
				throw new ERR_HTTP2_NO_SOCKET_MANIPULATION();
			default: {
				const socket = session[kSavedSocket];
				const value = socket[prop];
				return typeof value === 'function' ? value.bind(socket) : value;
			}
		}
	},
	getPrototypeOf(session) {
		return Reflect.getPrototypeOf(session[kSavedSocket]);
	},
	set(session, prop, value) {
		switch (prop) {
			case 'setTimeout':
				session.setTimeout = value;
				return true;
			case 'destroy':
			case 'emit':
			case 'end':
			case 'pause':
			case 'read':
			case 'resume':
			case 'write':
				throw new ERR_HTTP2_NO_SOCKET_MANIPULATION();
			default:
				session[kSavedSocket][prop] = value;
				return true;
		}
	}
};

module.exports = (session, socket) => {
	session[kSavedSocket] = socket;
	return new Proxy(session, proxySocketHandler);
};
